@@ -22,7 +22,7 @@
 ###         R functions
 ### 
 
-bootdistcens <- function (f, niter=1001)
+bootdistcens <- function (f, niter=1001, silent=TRUE)
 { 
     if (niter<10) 
         stop("niter must be an integer above 10")
@@ -33,22 +33,41 @@ bootdistcens <- function (f, niter=1001)
     numrow <- seq(1, n)
     rnumrow <- sample(numrow, size=niter*n, replace=TRUE)
     dim(rnumrow) <- c(n, niter)
-    start <- f$estimate
+    start <- as.list(f$estimate) #a named vector is no longer is accepted as starting values.
+    if(is.function(f$fix.arg.fun))
+      fix.arg <- f$fix.arg.fun
+    else 
+      fix.arg <- f$fix.arg
     if (is.null(f$dots))
-        funcmle <- function(iter) {
-        mle <- do.call(mledist, list(data=data.frame(left=f$censdata[rnumrow[, iter], ]$left, 
+    {
+      funcmle <- function(iter) {
+        mle <- try(do.call(mledist, list(data=data.frame(left=f$censdata[rnumrow[, iter], ]$left, 
             right=f$censdata[rnumrow[, iter], ]$right), distr=f$distname, start=start, 
-            fix.arg=f$fix.arg))
-        return(c(mle$estimate, mle$convergence))
+            fix.arg=fix.arg)), silent=silent)
+        if(inherits(mle, "try-error"))
+          return(c(rep(NA, length(start)), 100))
+        else
+          return(c(mle$estimate, mle$convergence))
+        
         }
-    else
-        funcmle <- function(iter) {
-        mle <- do.call(mledist, c(list(data=data.frame(left=f$censdata[rnumrow[, iter], ]$left, 
+    }else
+    {
+      funcmle <- function(iter) {
+        mle <- try(do.call(mledist, c(list(data=data.frame(left=f$censdata[rnumrow[, iter], ]$left, 
             right=f$censdata[rnumrow[, iter], ]$right), distr=f$distname, start=start), 
-            fix.arg=f$fix.arg, f$dots))
-        return(c(mle$estimate, mle$convergence))
+            fix.arg=fix.arg, f$dots)), silent=silent)
+        if(inherits(mle, "try-error"))
+          return(c(rep(NA, length(start)), 100))
+        else
+          return(c(mle$estimate, mle$convergence))
         }
+    }
+    owarn <- getOption("warn")
+    oerr <- getOption("show.error.messages")
+    options(warn=ifelse(silent, -1, 0), show.error.messages=!silent)
     resboot <- sapply(1:niter, funcmle)
+    options(warn=owarn, show.error.messages=oerr)
+    
     rownames(resboot) <- c(names(start), "convergence")
     if (length(resboot[, 1])>2) {
         estim <- data.frame(t(resboot)[, -length(resboot[, 1])])
